@@ -27,6 +27,13 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.78  2008/04/02 03:06:21  jsedwards
+-- Added file locking.
+--
+-- Revision 1.77  2008/03/24 05:04:10  jsedwards
+-- Change to not close private (or public) private descriptor if it is not
+-- open.
+--
 -- Revision 1.76  2008/02/25 14:31:20  jsedwards
 -- Changed nwos_allocate_new_chunk function to allow creating a new chunk for
 -- any reference and not just greater than the existing references.
@@ -607,6 +614,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
     uint16 right;
     uint16 save;
     bool compressed_file = false;
+    struct flock lock;
 #ifndef NO_BACKUP
     const char* backup_dir;
 #endif
@@ -661,6 +669,19 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 	exit(1);
     }
 
+#ifdef PUBLIC_MODE
+    lock.l_type = F_WRLCK;
+    lock.l_whence = SEEK_SET;
+    lock.l_start = 0;
+    lock.l_len = 0;
+
+    if (fcntl(public_file_desc, F_SETLK, &lock) != 0)
+    {
+	perror(nwos_public_path);
+	exit(1);
+    }
+#endif
+
     if (read(public_file_desc, &disk_header, sizeof(disk_header)) != sizeof(disk_header))
     {
 	snprintf(log_msg, sizeof(log_msg), "reading disk header from: %s", nwos_public_path);
@@ -713,6 +734,18 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 	    exit(1);
 	}
 
+	lock.l_type = read_only ? F_RDLCK : F_WRLCK;
+	lock.l_whence = SEEK_SET;
+	lock.l_start = 0;
+	lock.l_len = 0;
+
+	if (fcntl(private_file_desc, F_SETLK, &lock) != 0)
+	{
+	    perror(nwos_private_path);
+	    exit(1);
+	}
+
+
 	if (compressed_file)
 	{
 	    index_compressed_file();   /* build an index into the compressed file */
@@ -973,7 +1006,7 @@ void nwos_terminate_disk_io()
 	nwos_log("  No changes made");
     }
 
-    if (close(private_file_desc) < 0)
+    if (private_file_desc > 0 && close(private_file_desc) < 0)
     {
 	perror(nwos_private_path);
     }
@@ -985,7 +1018,7 @@ void nwos_terminate_disk_io()
     nwos_terminate_backup(modified, &disk_header);
 #endif
 
-    if (close(public_file_desc) < 0)
+    if (public_file_desc > 0 && close(public_file_desc) < 0)
     {
 	perror(nwos_public_path);
     }
