@@ -27,6 +27,17 @@
 --
 --
 -- $Log: expand_sparse.c,v $
+-- Revision 1.29  2008/03/16 14:12:39  jsedwards
+-- Fixed problem where it would print warning about compatible compressed file
+-- version even when it was the correct version.
+--
+-- Revision 1.28  2008/03/16 00:21:29  jsedwards
+-- Fix comparison with OLDEST_COMPATIBLE_COMPRESSED_VERSION.
+--
+-- Revision 1.27  2008/03/15 22:17:19  jsedwards
+-- Changed to use OLDEST_COMPATIBLE_COMPRESSED_VERSION define to determine if
+-- the compressed files is compatible.
+--
 -- Revision 1.26  2008/02/06 03:28:22  jsedwards
 -- Changed to call initialize_objectify with the DEFAULT_FILE (NULL) parameter
 -- instead of the storage file so that it won't think that it is a compressed
@@ -267,26 +278,37 @@ int main(int argc, char* argv[])
 
     /* Version 0023, 0024 and 0025 archives are identical so allow expanding 0023 or 0024 */
 
-    if (memcmp(&block[4], "0023", 4) == 0 || memcmp(&block[4], "0024", 4) == 0 || memcmp(&block[4], "0025", 4) == 0)
+    if (memcmp(VERSION_STRING, &block[4], 4) != 0)
     {
-	fprintf(stderr, "WARNING: compressed file is version 002%c, is compatible.\n", block[7]);
-	printf("WARNING: compressed file is version 002%c, is compatible.\n", block[7]);
-
-	sleep(5);
-    }
-    else
-    {
-	if (memcmp(VERSION_STRING, &block[4], 4) != 0)
+	if (memcmp(VERSION_STRING, &block[4], 4) < 0)
 	{
-	    printf("Version mismatch - disk: %s   file: %c%c%c%c\n",
-		   VERSION_STRING,
-		   compressed_header.version_string[0],
-		   compressed_header.version_string[1],
-		   compressed_header.version_string[2],
-		   compressed_header.version_string[3]);
+	    printf("ERROR: compressed file is version: %c%c%c%c, which newer than this version of software.\n",
+		   block[4], block[5], block[6], block[7]);
+	    printf("       Please upgrade your software to the latest version\n");
 	    fclose(ifp);
 	    exit(1);
 	}
+	else if (memcmp(OLDEST_COMPATIBLE_COMPRESSED_VERSION, &block[4], 4) <= 0)
+	{
+	    fprintf(stderr, "WARNING: compressed file is version 00%c%c, is compatible.\n", block[6], block[7]);
+	    printf("WARNING: compressed file is version 00%c%c, is compatible.\n", block[6], block[7]);
+
+	    sleep(10);
+	}
+	else
+	{
+	    printf("ERROR: compressed file is version: %c%c%c%c, which is incompatible.\n",
+		   block[4], block[5], block[6], block[7]);
+	    fclose(ifp);
+	    exit(1);
+	}
+
+	/* if old versions need special handling put it here and remove braces below */ 
+    }
+    else
+    {
+	printf("compressed file version: %c%c%c%c\n", block[4], block[5], block[6], block[7]);
+	sleep(5);
 
     }  /* remove these two braces if the the blocks and chunks */
     {  /* need to be read differently for the old file format */ 
@@ -297,11 +319,6 @@ int main(int argc, char* argv[])
 	nwos_4_uint8_to_uint32(compressed_header.used_blocks, &used_private_blocks);
 	nwos_4_uint8_to_uint32(compressed_header.used_chunks, &used_private_chunks);
 
-	printf("compressed file version: %c%c%c%c\n",
-	       compressed_header.version_string[0],
-	       compressed_header.version_string[1],
-	       compressed_header.version_string[2],
-	       compressed_header.version_string[3]);
     }
 
     printf("   total_private_blocks: %u  used_private_blocks: %u\n",
