@@ -1,23 +1,35 @@
-/*             This file is part of the New World OS project
---                Copyright (C) 2005, 2006  QRW Software
+/*
+--             This file is part of the New World OS project
+--                 Copyright (C) 2005-2007  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
 --
--- NWOS is free software;  you can redistribute it and/or modify it under the
--- terms of the GNU General Public License  as published by the Free Software
--- Foundation; either version 2, or (at your option) any later version.  This
--- software is distributed with the hope that it will be useful,  but WITHOUT
--- ANY WARRANTY;  without  even the  implied warranty  of MERCHANTABILITY  or
--- FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License for
--- more  details.  You should have received a copy  of the GNU General Public
--- License along with this package;  see the file LICENSE.  If not, write to:
+--   This program is free software: you can redistribute it and/or modify
+--   it under the terms of the GNU General Public License as published by
+--   the Free Software Foundation, either version 3 of the License, or
+--   (at your option) any later version.
+--
+--   This program is distributed in the hope that it will be useful,
+--   but WITHOUT ANY WARRANTY; without even the implied warranty of
+--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+--   GNU General Public License for more details.
+--
+--   You should have received a copy of the GNU General Public License
+--   along with this program, in the file LICENSE.  If not, see 
+--   <http://www.gnu.org/licenses/>.
+--
+--   You can also contact me via paper mail at:
+--
+--      QRW Software
+--      P.O. Box 27511
+--      Salt Lake City, UT 84127-0511, USA.
 --
---      Free Software Foundation, Inc.
---      59 Temple Place - Suite 330
---      Boston, MA 02111-1307, USA.
 --
 -- $Log: name.c,v $
+-- Revision 1.33  2007/07/01 19:44:12  jsedwards
+-- Upgrade to GPLv3.
+--
 -- Revision 1.32  2007/06/03 17:32:38  jsedwards
 -- Fix so when converting to string it changes the third letter ot upper case
 -- if the first two letters are "Mc" or "O'".
