@@ -27,6 +27,18 @@
 --
 --
 -- $Log: objectify.c,v $
+-- Revision 1.148  2008/03/20 13:13:50  jsedwards
+-- Added nwos_in_public_mode function.
+--
+-- Revision 1.147  2008/03/15 17:10:15  jsedwards
+-- Added 'nwos_get_object_class_without_update' function that gets an object's
+-- class without attempting to update it to the latest revision.  This avoids
+-- any nasty recursion problems.
+--
+-- Revision 1.146  2008/03/14 12:33:25  jsedwards
+-- Added get_file_002_reference function and modified nwos_get_object_class
+-- and nwos_read_object_from_disk to check for revision 002 FILE objects.
+--
 -- Revision 1.145  2008/02/27 05:48:32  jsedwards
 -- Change so that if no blowfish, linear, and serial values are specified
 -- (as is the case for expand_sparse) it doesn't try to initialize the
@@ -1171,6 +1183,12 @@ void nwos_terminate_objectify()
 }
 
 
+bool nwos_in_public_mode()
+{
+    return is_void_reference(&nwos_private_class_definition_class_ref);
+}
+
+
 /*******************************************************************/
 /* Reference utility functions: conversion, testing and generation */
 /*******************************************************************/
@@ -1517,7 +1535,7 @@ static ObjRef* get_file_class_reference()
 static ObjRef* get_file_001_reference()
 {
     static ObjRef result;
-    static bool inside_already = false;
+    static volatile bool inside_already = false;
     ObjRef class_ref;
     EveryObject header;
 
@@ -1562,6 +1580,57 @@ static ObjRef* get_file_001_reference()
 
     return &result;
 }
+
+static ObjRef* get_file_002_reference()
+{
+    static ObjRef result;
+    static volatile bool inside_already = false;
+    ObjRef class_ref;
+    EveryObject header;
+
+    if (is_void_reference(&result))
+    {
+        /* can't do anything until everything is initialized */
+	if (nwos_ready && !inside_already)
+	{
+	    inside_already = true;   /* in case we get called recursively */
+
+#ifdef PUBLIC_MODE
+	    nwos_find_public_class_definition("FILE", &class_ref);
+	    nwos_read_object_headers_from_disk(&class_ref, &header);
+	    assert(!is_void_reference(&header.object.prev_version));  /* there has to be at least one */
+#else
+	    /* in case there isn't an old one, make something non-void so won't rerun this each time */
+	    result.word = 0xffffffff;
+
+	    if (nwos_find_private_class_definition("FILE", &class_ref))
+	    {
+		nwos_read_object_headers_from_disk(&class_ref, &header);
+
+		if (is_void_reference(&header.object.prev_version))  /* updated class not created yet */
+		{
+		    copy_reference(&result, &class_ref);
+		}
+	    }
+	    else
+	    {
+		void_reference(&header.object.prev_version);
+	    }
+#endif
+	    while(!is_void_reference(&header.object.prev_version))
+	    {
+		copy_reference(&result, &header.object.prev_version);
+		nwos_read_object_headers_from_disk(&result, &header);
+	    }
+
+	    copy_reference(&result, &header.object.next_version);
+
+	    inside_already = false;
+	}
+    }
+
+    return &result;
+}
 #endif
 
 
@@ -1661,6 +1730,11 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 	check_object_data(ref, (uint8*)object, sizeof(C_struct_File_001), header->common.data_chksum);
 	nwos_update_file_001_object_to_current(object, size);
     }
+    else if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, get_file_002_reference()))
+    {
+	check_object_data(ref, (uint8*)object, sizeof(C_struct_File_002), header->common.data_chksum);
+	nwos_update_file_002_object_to_current(object, size);
+    }
     else
 #endif
 #endif
@@ -1964,7 +2038,11 @@ void nwos_write_public_object_to_disk(ObjRef* ref, void* object, size_t size)
 /* Class stuff */
 /***************/
 
-void nwos_get_object_class(ObjRef* obj, ObjRef* object_class)
+/*--------------------------------------------------------------------------------*/
+/* Get an object's class reference, without updating to the latest class revision */
+/*--------------------------------------------------------------------------------*/
+
+void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class)
 {
     EveryObject header;
     uint8 ivec[IVEC_SIZE];
@@ -1984,16 +2062,28 @@ void nwos_get_object_class(ObjRef* obj, ObjRef* object_class)
 	check_object_header(obj, &header.object, header.common.header_chksum);
     }
 
+    copy_reference(object_class, &header.common.class_definition);
+}
+
+
+/*-------------------------------------------------------------------------*/
+/* Get an object's class reference and update to the latest class revision */
+/*-------------------------------------------------------------------------*/
+
+void nwos_get_object_class(ObjRef* obj, ObjRef* object_class)
+{
+    nwos_get_object_class_without_update(obj, object_class);
+
 #ifndef USE_PREDEFINED_STRUCTS
-    if (nwos_reference_type(obj) == Private_Reference && is_same_object(&header.common.class_definition, get_file_001_reference()))
+    if (nwos_reference_type(obj) == Private_Reference && is_same_object(object_class, get_file_001_reference()))
     {
 	copy_reference(object_class, get_file_class_reference());
     }
-    else
-#endif
+    else if (nwos_reference_type(obj) == Private_Reference && is_same_object(object_class, get_file_002_reference()))
     {
-	copy_reference(object_class, &header.common.class_definition);
+	copy_reference(object_class, get_file_class_reference());
     }
+#endif
 }
 
 
