@@ -27,6 +27,20 @@
 --
 --
 -- $Log: objectify.h,v $
+-- Revision 1.135  2008/03/20 13:09:40  jsedwards
+-- Added external declaration for nwos_in_public_mode().
+--
+-- Revision 1.134  2008/03/15 22:15:42  jsedwards
+-- Added SHA256_Sum_Mismatch and SHA512_Sum_Mismatch to CheckFileResult
+-- enumeration.
+--
+-- Revision 1.133  2008/03/15 17:06:14  jsedwards
+-- Added external declaration for nwos_get_object_class_without_update.
+--
+-- Revision 1.132  2008/03/14 13:15:39  jsedwards
+-- Added define for SHA512_DIGEST_SIZE and external declaration for
+-- nwos_log_sha512sum.
+--
 -- Revision 1.131  2008/02/16 17:25:09  jsedwards
 -- Added external declaration for new nwos_add_info_to_existing_file function
 -- in file.c.
@@ -872,17 +886,20 @@ extern void   nwos_encode_variable_sized_count(uint32 count, uint8 result[4]);
 #define MD5_DIGEST_SIZE    16
 #define SHA1_DIGEST_SIZE   20
 #define SHA256_DIGEST_SIZE 32
+#define SHA512_DIGEST_SIZE 64
 
 extern void nwos_log(char* str);
 extern void nwos_log_arguments(int argc, char* argv[]);
 extern void nwos_log_md5sum(char* file_name, uint8 md5_digest[MD5_DIGEST_SIZE]);
 extern void nwos_log_sha1sum(char* file_name, uint8 sha1_digest[SHA1_DIGEST_SIZE]);
 extern void nwos_log_sha256sum(char* file_name, uint8 sha256_digest[SHA256_DIGEST_SIZE]);
+extern void nwos_log_sha512sum(char* file_name, uint8 sha512_digest[SHA512_DIGEST_SIZE]);
 
 extern void nwos_initialize_objectify(uint8 bf_key[16], uint32 linear, uint32 serial, AccessType type, const char* path);
 extern void nwos_terminate_objectify();
 extern void nwos_set_root_object(ObjRef* ref);
 extern void nwos_flush_bit_maps();
+extern bool nwos_in_public_mode();
 
 extern size_t nwos_reference_list_size(ObjRef* ref);
 extern ReferenceList* nwos_malloc_reference_list(ObjRef* ref);
@@ -890,6 +907,7 @@ extern void nwos_free_reference_list(ReferenceList* ref_list);
 
 extern Reference_Type nwos_reference_type(ObjRef* ref);
 extern void nwos_get_object_class(ObjRef* obj, ObjRef* object_class);
+extern void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class);
 extern size_t nwos_get_object_size(void* object);
 extern void nwos_restart_id_generation();
 extern void nwos_generate_new_id(ObjRef* ref);
@@ -1014,7 +1032,7 @@ extern void nwos_list_public_manufacturers();
 
 
 /* file stuff */
-typedef enum { File_Not_Found, Checksums_Match, MD5_Sum_Mismatch, SHA1_Sum_Mismatch, Checksums_Mismatch } CheckFileResult;
+typedef enum { File_Not_Found, Checksums_Match, MD5_Sum_Mismatch, SHA1_Sum_Mismatch, SHA256_Sum_Mismatch, SHA512_Sum_Mismatch, Checksums_Mismatch } CheckFileResult;
 
 extern bool nwos_url_to_string(ObjRef* ref, char* string, size_t size);
 
