@@ -27,6 +27,9 @@
 --
 --
 -- $Log: objectify_private.h,v $
+-- Revision 1.64  2008/03/14 12:16:26  jsedwards
+-- Added external reference for nwos_update_file_002_object_to_current.
+--
 -- Revision 1.63  2008/02/05 04:21:06  jsedwards
 -- Added external declaration for nwos_get_log_file_path.
 --
@@ -488,6 +491,7 @@ extern ObjCreateResult nwos_create_url(char* url, ObjRef* ref);
 extern void nwos_create_public_manufacturer(char* brand_name, char* acronym, char* url_list[], ObjRef* ref);
 
 extern void nwos_update_file_001_object_to_current(void* object, size_t size);
+extern void nwos_update_file_002_object_to_current(void* object, size_t size);
 
 #endif
 
