@@ -1,23 +1,93 @@
-/*             This file is part of the New World OS project
+/*
+--             This file is part of the New World OS project
 --                 Copyright (C) 2004-2007  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
 --
--- NWOS is free software;  you can redistribute it and/or modify it under the
--- terms of the GNU General Public License  as published by the Free Software
--- Foundation; either version 2, or (at your option) any later version.  This
--- software is distributed with the hope that it will be useful,  but WITHOUT
--- ANY WARRANTY;  without  even the  implied warranty  of MERCHANTABILITY  or
--- FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License for
--- more  details.  You should have received a copy  of the GNU General Public
--- License along with this package;  see the file LICENSE.  If not, write to:
+--   This program is free software: you can redistribute it and/or modify
+--   it under the terms of the GNU General Public License as published by
+--   the Free Software Foundation, either version 3 of the License, or
+--   (at your option) any later version.
+--
+--   This program is distributed in the hope that it will be useful,
+--   but WITHOUT ANY WARRANTY; without even the implied warranty of
+--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+--   GNU General Public License for more details.
+--
+--   You should have received a copy of the GNU General Public License
+--   along with this program, in the file LICENSE.  If not, see 
+--   <http://www.gnu.org/licenses/>.
+--
+--   You can also contact me via paper mail at:
+--
+--      QRW Software
+--      P.O. Box 27511
+--      Salt Lake City, UT 84127-0511, USA.
 --
---      Free Software Foundation, Inc.
---      59 Temple Place - Suite 330
---      Boston, MA 02111-1307, USA.
 --
 -- $Log: reference_list.c,v $
+-- Revision 1.31  2007/09/16 15:00:48  jsedwards
+-- Fixed case where entry to be removed is in first block and there are two
+-- or more blocks in the list.
+--
+-- Revision 1.30  2007/09/16 13:30:50  jsedwards
+-- Added function header for remove_from_reference_list that explains the
+-- different cases, added and changed some comments in the code.  NO code
+-- changes.
+--
+-- Revision 1.29  2007/09/16 03:53:56  jsedwards
+-- Added code to handle moving entries down one and rewriting intermediate
+-- blocks.
+--
+-- Revision 1.28  2007/09/15 20:53:45  jsedwards
+-- Changes to handle removing a block when an entry is removed in the previous
+-- block and there is only one entry in the last block.
+--
+-- Revision 1.27  2007/09/15 19:36:56  jsedwards
+-- Added code to deal with removing block because there was only one entry in
+-- it, for blocks beyond the second block.
+--
+-- Revision 1.26  2007/09/15 18:59:26  jsedwards
+-- Moved ivec out of Block_Cache and into First_Block and Extra_Block
+-- structures so we have the ivec for every block.  Needed when removing
+-- entries from a reference list and need to write blocks other than the last.
+--
+-- Revision 1.25  2007/09/15 16:32:59  jsedwards
+-- Added code in remove_from_reference_list to deal with special case where
+-- there is two blocks in the list, there is only one entry in the second
+-- list and that entry is the one that has to be removed.
+--
+-- Revision 1.24  2007/09/15 14:04:36  jsedwards
+-- Added code to remove_from_reference_list to deal with the entry being in
+-- the first block if there is only one entry after the first block and so
+-- the second block can be eliminated.
+--
+-- Revision 1.23  2007/09/14 13:27:32  jsedwards
+-- Removed unused ivec variable in write_reference_list.
+--
+-- Revision 1.22  2007/09/14 12:56:12  jsedwards
+-- Changed so that when removing entry from reference list that only has one
+-- block, it moves them down by one instead of copying the last over it.
+--
+-- Revision 1.21  2007/09/14 00:07:07  jsedwards
+-- Added code to nwos_remove_from_reference_list to remove entry as long as
+-- it is in last block.
+--
+-- Revision 1.20  2007/08/30 03:58:01  jsedwards
+-- Moved #endif to the correct place (mistake in previous check-in).
+--
+-- Revision 1.19  2007/08/29 04:20:17  jsedwards
+-- Put #ifdefs around two more log messages of add to references so normally
+-- the are not printed.
+--
+-- Revision 1.18  2007/07/11 02:45:08  jsedwards
+-- Fix bug in create_reference_list where not enough memory was allocated for
+-- the first block.
+--
+-- Revision 1.17  2007/07/01 19:44:12  jsedwards
+-- Upgrade to GPLv3.
+--
 -- Revision 1.16  2007/06/30 14:14:01  jsedwards
 -- Added #ifdef around logging of add_to_reference_list, so it is off by default.
 --
@@ -87,7 +157,6 @@ typedef struct {
   int num_refs;
   Ref_List_First_Block* first_block;
   int age;
-  uint8 ivec[IVEC_SIZE];
 } Block_Cache;
 
 
@@ -145,8 +214,8 @@ static void read_ref_list_into_cache(Block_Cache* cache)
     int i;
     int seq;
 
-    assert(sizeof(Ref_List_First_Block) == FILE_BLOCK_SIZE + sizeof(void*));
-    assert(sizeof(Ref_List_Extra_Block) == FILE_BLOCK_SIZE + sizeof(void*));
+    assert(sizeof(Ref_List_First_Block) == FILE_BLOCK_SIZE + sizeof(void*) + IVEC_SIZE);
+    assert(sizeof(Ref_List_Extra_Block) == FILE_BLOCK_SIZE + sizeof(void*) + IVEC_SIZE);
 
     memset(ivec, 0, sizeof(ivec));
 
@@ -154,7 +223,7 @@ static void read_ref_list_into_cache(Block_Cache* cache)
     assert(cache->first_block != NULL);
 
     memset(ivec, 0, sizeof(ivec));
-    memcpy(cache->ivec, ivec, sizeof(cache->ivec));
+    memcpy(cache->first_block->ivec, ivec, sizeof(cache->first_block->ivec));
 
     cache->first_block->next_block_ptr = NULL;
 
@@ -195,7 +264,7 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 	next_ptr->next_block_ptr = NULL;
 
-	memcpy(cache->ivec, ivec, sizeof(cache->ivec));  /* save this ivec in case this block has to be written back to disk */
+	memcpy(next_ptr->ivec, ivec, sizeof(next_ptr->ivec));  /* save this ivec in case this block has to be written back to disk */
 
 	nwos_read_object_from_disk_and_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
 
@@ -228,7 +297,6 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 static void write_reference_list(Block_Cache* cache)
 {
-    uint8 ivec[IVEC_SIZE];
     Ref_List_Extra_Block *block = NULL;
     int running_total;
     int refs_remaining;
@@ -241,8 +309,6 @@ static void write_reference_list(Block_Cache* cache)
 
     encrypted = (nwos_reference_type(&cache->ref) == Private_Reference);
 
-    memset(ivec, 0, sizeof(ivec));
-
     if (cache->num_refs <= MAX_REFS_IN_REF_LIST)    /* there is just one block */
     {
 	assert(cache->first_block->next_block_ptr == NULL);  /* the next block pointer should be null */
@@ -374,7 +440,7 @@ static void write_reference_list(Block_Cache* cache)
 
 	/* printf("writing block: %02x%02x%02x%02x\n", ref.id[0], ref.id[1], ref.id[2], ref.id[3]); */
 
-	nwos_write_object_to_disk_and_encrypt(&ref, &block->dirty, FILE_BLOCK_SIZE, cache->ivec, nwos_random_sequence[seq]);
+	nwos_write_object_to_disk_and_encrypt(&ref, &block->dirty, FILE_BLOCK_SIZE, block->ivec, nwos_random_sequence[seq]);
 
 	nwos_free(block);
     }
@@ -547,15 +613,15 @@ void nwos_create_reference_list_with_existing_id(ObjRef* for_obj, ObjRef* ref_li
     ref_list_tick++;
     cache->age = ref_list_tick;
 
-    cache->first_block = nwos_malloc(FILE_BLOCK_SIZE);
+    cache->first_block = nwos_malloc(sizeof(Ref_List_First_Block));
 
     assert(cache->first_block != NULL);
 
-    memset(cache->first_block, 0, FILE_BLOCK_SIZE);  /* zero it out */
+    memset(cache->first_block, 0, sizeof(Ref_List_First_Block));  /* zero it out */
 
     cache->num_refs = 0;
 
-    memset(cache->ivec, 0, sizeof(cache->ivec));
+    memset(cache->first_block->ivec, 0, sizeof(cache->first_block->ivec));
 
     nwos_fill_in_common_header(&cache->first_block->list.common_header, &cache->ref, &nwos_reference_list_class_ref);
 
@@ -571,6 +637,7 @@ void nwos_create_reference_list_with_existing_id(ObjRef* for_obj, ObjRef* ref_li
 
     assert(cache->num_refs == 0);
     assert(cache->first_block != NULL);
+    assert(cache->first_block->next_block_ptr == NULL);
     assert(is_void_reference(&cache->first_block->refs[cache->num_refs]));
 }
 
@@ -605,7 +672,9 @@ void nwos_add_to_references(ObjRef* ref, ObjRef* obj)
 
 void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 {
+#ifdef LOG_REFERENCE_LIST
     char log_msg[128];
+#endif
     Block_Cache* cache;
     int running_total;
     int refs_remaining;
@@ -661,7 +730,7 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 				 (cache->num_refs + 1) * sizeof(ObjRef),               /* include void ptr or next_block_ref */
 				 cache->first_block->list.common_header.data_chksum);
 
-	    nwos_write_object_to_disk_and_encrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, cache->ivec, nwos_random_sequence[0]);
+	    nwos_write_object_to_disk_and_encrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]);
 
 	    next_ptr = nwos_malloc(sizeof(Ref_List_Extra_Block));
 
@@ -671,12 +740,16 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 	    next_ptr->next_block_ptr = NULL;
 
+	    memcpy(next_ptr->ivec, ivec, sizeof(next_ptr->ivec));
+
+#ifdef LOG_REFERENCE_LIST
 	    snprintf(log_msg, sizeof(log_msg), "add_reference - first_block ref: %02x%02x%02x%02x obj: %p  next_block ref: %02x%02x%02x%02x obj: %p",
 		   cache->ref.id[0], cache->ref.id[2], cache->ref.id[2], cache->ref.id[3], cache->first_block,
 		   cache->first_block->next_block_ref.id[0], cache->first_block->next_block_ref.id[1], 
 		   cache->first_block->next_block_ref.id[2], cache->first_block->next_block_ref.id[3], 
 		   cache->first_block->next_block_ptr);
 	    nwos_log(log_msg);
+#endif
 
 	    next_ptr->dirty = -1;     /* mark it as dirty */
 	    memcpy(&next_ptr->id, &cache->first_block->next_block_ref, sizeof(ObjRef));
@@ -725,7 +798,9 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 					 (refs_remaining + 1) * sizeof(ObjRef),        /* include void ptr or next_block_ref */
 					 next_ptr->checksum);
 
-		    nwos_write_object_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, cache->ivec, nwos_random_sequence[seq]);
+		    memcpy(ivec, next_ptr->ivec, sizeof(ivec));
+
+		    nwos_write_object_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
 
 		    next_ptr->next_block_ptr = nwos_malloc(sizeof(Ref_List_Extra_Block));
 
@@ -733,6 +808,9 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 		    next_ptr->next_block_ptr->next_block_ptr = NULL;
 
+		    memcpy(next_ptr->next_block_ptr->ivec, ivec, sizeof(next_ptr->next_block_ptr->ivec));
+
+#ifdef LOG_REFERENCE_LIST
 	    snprintf(log_msg, sizeof(log_msg), "add_reference - next_block ref: %02x%02x%02x%02x obj: %p  next_block ref: %02x%02x%02x%02x obj: %p",
 		   next_ptr->id.id[0], next_ptr->id.id[2], next_ptr->id.id[2], next_ptr->id.id[3], 
 		   next_ptr,
@@ -740,6 +818,7 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 		   next_ptr->next_block_ref.id[2], next_ptr->next_block_ref.id[3], 
 		   next_ptr->next_block_ptr);
 	    nwos_log(log_msg);
+#endif
 
 		    next_ptr->next_block_ptr->dirty = -1;     /* mark it as dirty */
 		    memcpy(&next_ptr->next_block_ptr->id, &next_ptr->next_block_ref, sizeof(ObjRef));
@@ -788,13 +867,35 @@ void nwos_remove_from_references(ObjRef* ref, ObjRef* obj)
 
 
 
-/* Add a new reference to the reference list specified (ref_list is the id of the reference list itself). */
+/*-------------------------------------------------------------------------------------------------------
+ Remove a reference from the reference list specified (ref_list is the id of the reference list itself).
+ 
+  Since the first block of a reference list is different than the subsequent blocks,
+  there are some special cases which arise:
+
+    1. There is only one block.
+    2. There are two blocks, but the second block only has 1 entry
+    3. There are multiple blocks but the entry to be deleted is in the first.
+    4. There are two blocks, the entry is in the first, and the second only has one entry.
+
+  After we are past the first blocker there are four cases:
+
+    5. The entry is in the last block, no other blocks need to be modified.
+    6. The entry is the only entry in the last block, the last block needs to be eliminated.
+    7. The entry is not in the last block, blocks from the entry to the end need to be written.
+    8. The entry is not in the last, and the last block has only one entry and needs to be removed.
+-------------------------------------------------------------------------------------------------------*/
 
 void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 {
     char log_msg[128];
     Block_Cache* cache;
+    Ref_List_Extra_Block* next_ptr;
+    Ref_List_Extra_Block* prev_ptr = NULL;
+    int count;
     int i;
+    int seq;
+    uint8 ivec[IVEC_SIZE];
 
 #ifdef PUBLIC_MODE
     assert(nwos_reference_type(ref) == Public_Reference);
@@ -816,7 +917,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 	    cache->num_refs);
     nwos_log(log_msg);
 
-    if (cache->num_refs <= MAX_REFS_IN_REF_LIST)    /* there is just one block */
+    if (cache->num_refs <= MAX_REFS_IN_REF_LIST)    /* case #1 - there is only one block */
     {
 	assert(cache->first_block->next_block_ptr == NULL);         /* the next block pointer should be null */
 	assert(is_void_reference(&cache->first_block->refs[cache->num_refs]));
@@ -832,16 +933,227 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 		
 	    assert(i < cache->num_refs);    /* make sure it was in the list */
 
-	    copy_reference(&cache->first_block->refs[i], &cache->first_block->refs[cache->num_refs]);
+	    while ((i + 1) < MAX_REFS_IN_REF_LIST && !is_void_reference(&cache->first_block->refs[i+1]))
+	    {
+		copy_reference(&cache->first_block->refs[i], &cache->first_block->refs[i+1]);
+		i++;
+	    }
+
+	    assert(i == cache->num_refs);
 	}
 
 	void_reference(&cache->first_block->refs[cache->num_refs]);
 	cache->first_block->list.common_header.flags = 0xffffffff;     /* mark it as dirty */
     }
-    else
+    else   /* cases 2 through 8 - more than one block */
     {
-	bool this_version_cannot_handle_removing_from_long_reference_list = false;
-	assert(this_version_cannot_handle_removing_from_long_reference_list);
+	assert(!is_void_reference(&cache->first_block->next_block_ref));
+
+	for (i = 0; i < MAX_REFS_IN_REF_LIST; i++)
+	{
+	    if (is_same_object(ref, &cache->first_block->refs[i])) break;
+	}
+		
+	next_ptr = cache->first_block->next_block_ptr;
+	assert(next_ptr != NULL);         /* the next block pointer should not be null */
+
+	count = i;
+
+	if (i < MAX_REFS_IN_REF_LIST)  /* case 2 or 3 - is in the first and there are blocks after it */
+	{
+	    assert(is_same_object(ref, &cache->first_block->refs[i]));
+
+	    while (i + 1 < MAX_REFS_IN_REF_LIST)
+	    {
+		copy_reference(&cache->first_block->refs[i], &cache->first_block->refs[i + 1]);
+
+		count++;
+		i++;
+	    }
+
+	    assert (i == MAX_REFS_IN_REF_LIST - 1);
+
+	    copy_reference(&cache->first_block->refs[i], &next_ptr->refs[0]);
+	    count++;
+
+	    if (cache->num_refs == (MAX_REFS_IN_REF_LIST + 1)) /* case 2 - don't need extra block anymore */
+	    {
+		nwos_clear_bit_in_map(nwos_hash_ref(&cache->first_block->next_block_ref));
+		void_reference(&cache->first_block->next_block_ref);
+
+		cache->first_block->next_block_ptr = NULL;
+
+		nwos_free(next_ptr);
+		next_ptr = NULL;    /* flag that we don't need to move anymore */
+
+		cache->first_block->list.common_header.flags = 0xffffffff;     /* mark it as dirty */
+
+		/* should not need to change ivec for first block, should always be zero */
+	    }
+	    else   /* case 3 - is in first block, and we need to deal with following blocks */
+	    {
+		nwos_crc32_calculate((uint8*) &cache->first_block->refs, 
+				     (MAX_REFS_IN_REF_LIST + 1) * sizeof(ObjRef),  /* include next_block_ref */
+				     cache->first_block->list.common_header.data_chksum);
+
+		memcpy(ivec, cache->first_block->ivec, sizeof(ivec));  /* should be all zeros */
+		nwos_write_object_to_disk_and_encrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]);
+
+		memcpy(next_ptr->ivec, ivec, sizeof(ivec));
+
+		i = 0;
+		seq = 1;
+	    }
+	}
+	else if (cache->num_refs == (MAX_REFS_IN_REF_LIST + 1))  /* case 4, two blocks with 1 entry in 2nd */
+	{
+	    assert(is_same_object(ref, &next_ptr->refs[0]));
+	    assert(is_void_reference(&next_ptr->refs[1]));
+
+	    nwos_clear_bit_in_map(nwos_hash_ref(&cache->first_block->next_block_ref));
+	    void_reference(&cache->first_block->next_block_ref);
+
+	    cache->first_block->next_block_ptr = NULL;
+
+	    nwos_free(next_ptr);
+	    next_ptr = NULL;    /* flag that we don't need to move anymore */
+
+	    cache->first_block->list.common_header.flags = 0xffffffff;     /* mark it as dirty */
+
+	    /* should not need to change ivec for first block, should always be zero */
+	}
+	else   /* cases 5 through 8 */
+	{
+	    seq = 1;
+
+	    while (true)
+	    {
+		assert (next_ptr != NULL);
+
+		for (i = 0; i < MAX_REFS_IN_SIDECAR; i++)
+		{
+		    assert(!is_void_reference(&next_ptr->refs[i]));
+
+		    if (is_same_object(ref, &next_ptr->refs[i])) break;
+
+		    count++;
+		}
+
+		if (i < MAX_REFS_IN_SIDECAR) break;
+
+		prev_ptr = next_ptr;
+
+		next_ptr = next_ptr->next_block_ptr;
+
+		seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+	    }
+	}	
+
+	if (next_ptr != NULL)   /* cases 3 and 5-8 */
+	{
+	    assert(i < MAX_REFS_IN_SIDECAR);
+
+	    if (i == 0 && is_void_reference(&next_ptr->refs[1]))  /* case 6 - only entry in this block */
+	    {
+		assert(is_same_object(ref, &next_ptr->refs[0]));
+		assert(prev_ptr != NULL);
+		assert(prev_ptr->next_block_ptr == next_ptr);
+		assert(is_same_object(&prev_ptr->next_block_ref, &next_ptr->id));
+
+		nwos_clear_bit_in_map(nwos_hash_ref(&next_ptr->id));
+		void_reference(&prev_ptr->next_block_ref);
+
+		prev_ptr->next_block_ptr = NULL;
+
+		nwos_free(next_ptr);
+
+		prev_ptr->dirty = -1;
+	    }
+	    else if (next_ptr->next_block_ptr == NULL)   /* case 5 - just need to remove it from the last block */
+	    {
+		while (!is_void_reference(&next_ptr->refs[i]))
+		{
+		    copy_reference(&next_ptr->refs[i], &next_ptr->refs[i+1]);
+
+		    count++;
+		    i++;
+		}
+
+		next_ptr->dirty = -1;     /* mark it as dirty */
+	    }
+	    else   /* case 7 or 8 */
+	    {
+		while (next_ptr->next_block_ptr != NULL)
+		{
+		    while ((i + 1) < MAX_REFS_IN_SIDECAR)
+		    {
+			copy_reference(&next_ptr->refs[i], &next_ptr->refs[i+1]);
+
+			count++;
+			i++;
+		    }
+
+		    copy_reference(&next_ptr->refs[i], &next_ptr->next_block_ptr->refs[0]);
+
+		    nwos_crc32_calculate((uint8*) &next_ptr->refs, 
+					 (MAX_REFS_IN_SIDECAR + 1) * sizeof(ObjRef),  /* include next_block_ref */
+					 next_ptr->checksum);
+
+		    memcpy(ivec, next_ptr->ivec, sizeof(ivec));
+		    nwos_write_object_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
+
+		    memcpy(next_ptr->next_block_ptr->ivec, ivec, sizeof(ivec));
+
+		    prev_ptr = next_ptr;
+
+		    next_ptr = next_ptr->next_block_ptr;
+
+		    seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+
+		    i = 0;
+		}
+
+		assert(prev_ptr->next_block_ptr == next_ptr);
+		assert(is_same_object(&prev_ptr->next_block_ref, &next_ptr->id));
+
+		if (!is_void_reference(&next_ptr->refs[0]) && is_void_reference(&next_ptr->refs[1])) /* case 8 - only one entry in last block */
+		{
+		    assert(prev_ptr != NULL);
+
+		    nwos_clear_bit_in_map(nwos_hash_ref(&prev_ptr->next_block_ref));
+		    void_reference(&prev_ptr->next_block_ref);
+
+		    nwos_free(prev_ptr->next_block_ptr);
+		    prev_ptr->next_block_ptr = NULL;
+
+		    prev_ptr->dirty = -1;     /* mark it as dirty */
+		}
+		else  /* case 7 - just need to finish last block */
+		{
+		    assert(!is_void_reference(&next_ptr->refs[i + 1]));
+
+		    while (!is_void_reference(&next_ptr->refs[i + 1]))
+		    {
+			copy_reference(&next_ptr->refs[i], &next_ptr->refs[i+1]);
+
+			count++;
+			i++;
+		    }
+
+		    void_reference(&next_ptr->refs[i]);
+
+		    next_ptr->dirty = -1;
+		}
+	    }
+
+	    next_ptr = NULL;
+	}
+
+	assert(next_ptr == NULL);
+
+	printf("num_refs: %d  count: %d\n", cache->num_refs, count);
+
+	cache->num_refs--;
     }
 }
 
