@@ -27,6 +27,10 @@
 --
 --
 -- $Log: time_stamp.h,v $
+-- Revision 1.9  2008/02/16 18:12:09  jsedwards
+-- Added new nwos_time_stamp_is_zero inline function to test to see if a
+-- time stamp is all zeros.
+--
 -- Revision 1.8  2007/10/22 03:53:26  jsedwards
 -- Added include of sys/time.h to define struct timeval on some platforms.
 --
@@ -115,6 +119,12 @@ extern void nwos_convert_timespec_to_time_stamp(struct timespec* ts, TimeStamp t
 extern void nwos_convert_time_stamp_to_timeval(TimeStamp time_stamp, struct timeval* tv);
 
 
+static inline bool nwos_time_stamp_is_zero(TimeStamp stamp)
+{
+    return (stamp[0] | stamp[1] | stamp[2] | stamp[3] | stamp[4] | stamp[5] | stamp[6] | stamp[7]) == 0;
+}
+
+
 static inline uint nwos_extract_year_from_time_stamp(TimeStamp stamp)
 {
     uint year = (stamp[0] << 4) | (stamp[1] >> 4);
