@@ -27,6 +27,9 @@
 --
 --
 -- $Log: verify_file.c,v $
+-- Revision 1.13  2008/03/20 14:03:50  jsedwards
+-- Added --public option.
+--
 -- Revision 1.12  2008/02/03 01:04:28  jsedwards
 -- Changed DEFAULT_TYPE_RO to READ_ONLY and changed path variable to const char
 -- instead of char.
@@ -92,6 +95,7 @@ int main(int argc, char* argv[])
     ObjRef ref;
     int i;
     bool do_remove = false;
+    bool do_public = false;
     MatchCode match = MatchAll;
     char* path = DEFAULT_FILE;
 
@@ -103,6 +107,10 @@ int main(int argc, char* argv[])
 	{
 	    do_remove = true;
 	}
+	else if (strcmp(argv[i], "--public") == 0)
+	{
+	    do_public = true;
+	}
 	else if (strcmp(argv[i], "--ignore-time") == 0)
 	{
 	    match = IgnoreTime;
@@ -121,17 +129,36 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
-    nwos_get_key_from_password(big_key, sizeof(big_key));
+    if (do_public && path != NULL)   /* both --public and --compressed were specified */
+    {
+	fprintf(stderr, "WARNING: because --public was specified --compressed file is ignored!\n");
+	path = NULL;
+    }
 
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
+    if (do_public)
+    {
+	nwos_initialize_objectify(NULL, 0, 0, PUBLIC, NULL);
 
-    nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, path);
+	root_object_ref.id[0] = 0;
+	root_object_ref.id[1] = 0;
+	root_object_ref.id[2] = 0;
+	root_object_ref.id[3] = 1;
 
-    nwos_set_root_object(&root_object_ref);
+	nwos_set_root_object(&root_object_ref);
+    }
+    else
+    {
+	nwos_get_key_from_password(big_key, sizeof(big_key));
 
+	memcpy(bf_key, big_key, 16);
+	linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
+	memcpy(root_object_ref.id, big_key+20, 4);
+	serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
+
+	nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, path);
+
+	nwos_set_root_object(&root_object_ref);
+    }
 
     for (/* use i as left by loop above */; i < argc; i++)
     {
